@@ -160,7 +160,7 @@ namespace apvlv
     return itr;
   }
 
-  ApvlvDir::ApvlvDir (int w, int h)
+  ApvlvDir::ApvlvDir (ApvlvView *view, int w, int h): ApvlvCore(view)
   {
     mReady = false;
 
@@ -192,8 +192,8 @@ namespace apvlv
     gtk_tree_view_column_pack_start (column, renderer, FALSE);
     gtk_tree_view_column_add_attribute (column, renderer0, "pixbuf", 1);
     gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
-    gtk_tree_view_column_set_sort_column_id (column, 2);
-    gtk_tree_view_column_set_sort_order (column, GTK_SORT_ASCENDING);
+    //gtk_tree_view_column_set_sort_column_id (column, 2);
+    //gtk_tree_view_column_set_sort_order (column, GTK_SORT_ASCENDING);
     gtk_tree_view_append_column (GTK_TREE_VIEW (mDirView), column);
     gtk_tree_view_column_clicked (column);
 
@@ -247,7 +247,7 @@ namespace apvlv
 	|| *path == '\0'
 	|| (rpath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL)) == NULL)
       {
-	gView->errormessage ("path error: %s", path ? path : "No path");
+	mView->errormessage ("path error: %s", path ? path : "No path");
 	return false;
       }
 
@@ -256,7 +256,7 @@ namespace apvlv
     g_free (rpath);
     if (ret < 0)
       {
-	gView->errormessage ("stat error: %d:%s", errno, strerror (errno));
+	mView->errormessage ("stat error: %d:%s", errno, strerror (errno));
 	return false;
       }
 
@@ -381,7 +381,7 @@ namespace apvlv
       case ':':
       case '/':
       case '?':
-	gView->promptcommand (key);
+	mView->promptcommand (key);
 	return NEED_MORE;
 
       case 'n':
@@ -460,12 +460,23 @@ namespace apvlv
 	return false;
       }
 
+    if (key == GDK_KEY_Return)
+      {
+        bool disable_content = false;
+        if (name == NULL)
+          {
+            name = filename();
+            disable_content = true;
+          }
+        return mView->newview (name, pn, disable_content);
+      }
+
     ApvlvCore *ndoc = NULL;
     if (name != NULL)
       {
 	if (gParams->valueb ("content"))
 	  {
-	    ndoc = new ApvlvDir (mWidth, mHeight);
+	    ndoc = new ApvlvDir (mView, mWidth, mHeight);
 	    if (!ndoc->loadfile (name))
 	      {
 		delete ndoc;
@@ -475,8 +486,9 @@ namespace apvlv
 
 	if (ndoc == NULL)
 	  {
+            DISPLAY_TYPE type = get_display_type_by_filename (name);
 	    ndoc =
-	      new ApvlvDoc (mWidth, mHeight, gParams->values ("zoom"),
+	      new ApvlvDoc (mView, type, mWidth, mHeight, gParams->values ("zoom"),
 			    gParams->valueb ("cache"));
 	    if (!ndoc->loadfile (name))
 	      {
@@ -487,8 +499,10 @@ namespace apvlv
       }
     else
       {
+        const char *name = filename ();
+        DISPLAY_TYPE type = get_display_type_by_filename (name);
 	ndoc =
-	  new ApvlvDoc (mWidth, mHeight, gParams->values ("zoom"),
+	  new ApvlvDoc (mView, type, mWidth, mHeight, gParams->values ("zoom"),
 			gParams->valueb ("cache"));
 	if (!ndoc->loadfile (filename ()))
 	  {
@@ -507,20 +521,16 @@ namespace apvlv
 	return false;
       }
 
-    gView->regloaded (ndoc);
+    mView->regloaded (ndoc);
 
     switch (key)
       {
-      case GDK_KEY_Return:
-	ApvlvWindow::currentWindow ()->setCore (ndoc);
-	break;
-
       case 'o':
 	ApvlvWindow::currentWindow ()->birth (false, ndoc);
 	break;
 
       case 't':
-	gView->newtab (ndoc);
+	mView->newtab (ndoc);
 	break;
 
       default:
@@ -674,7 +684,7 @@ namespace apvlv
     gtk_tree_model_get (GTK_TREE_MODEL (mStore), &mCurrentIter, 0, &info, -1);
     if (info == NULL || mDirNodes == NULL)
       {
-	gView->errormessage ("can't find word: '%s'", mSearchStr.c_str ());
+	mView->errormessage ("can't find word: '%s'", mSearchStr.c_str ());
 	return false;
       }
 
@@ -689,7 +699,7 @@ namespace apvlv
 
     if (list == NULL)
       {
-	gView->errormessage ("can't find word: '%s'", mSearchStr.c_str ());
+	mView->errormessage ("can't find word: '%s'", mSearchStr.c_str ());
 	return false;
       }
 
@@ -745,7 +755,7 @@ namespace apvlv
 
     if (list == NULL)
       {
-	gView->errormessage ("can't find word: '%s'", mSearchStr.c_str ());
+	mView->errormessage ("can't find word: '%s'", mSearchStr.c_str ());
 	return false;
       }
 
@@ -819,13 +829,13 @@ namespace apvlv
     for (unsigned int i = 0; i < AD_STATUS_SIZE; ++i)
       {
 #if GTK_CHECK_VERSION(3, 0, 0)
-        gtk_widget_override_color (mStlab[i],
-                                   (act) ? GTK_STATE_FLAG_ACTIVE:
-                                   GTK_STATE_FLAG_INSENSITIVE, NULL);
+        gtk_widget_set_state_flags (mStlab[i],
+				    (act) ? GTK_STATE_FLAG_ACTIVE:
+				    GTK_STATE_FLAG_INSENSITIVE, TRUE);
 #else
-	gtk_widget_modify_fg (mStlab[i],
-			      (act) ? GTK_STATE_ACTIVE :
-			      GTK_STATE_INSENSITIVE, NULL);
+        gtk_widget_modify_fg (mStlab[i],
+                              (act) ? GTK_STATE_ACTIVE:
+                              GTK_STATE_INSENSITIVE, NULL);
 #endif
       }
   }
@@ -901,6 +911,12 @@ namespace apvlv
 
     if (g_ascii_strncasecmp (basename + strlen (basename) - 4, ".pdf", 4)
 	!= 0
+	&& g_ascii_strncasecmp (basename + strlen (basename) - 4,
+				".htm", 4) != 0
+	&& g_ascii_strncasecmp (basename + strlen (basename) - 5,
+				".html", 5) != 0
+	&& g_ascii_strncasecmp (basename + strlen (basename) - 5,
+				".epub", 5) != 0
 #ifdef APVLV_WITH_DJVU
 	&& g_ascii_strncasecmp (basename + strlen (basename) - 5,
 				".djvu", 5) != 0
@@ -910,16 +926,6 @@ namespace apvlv
 #ifdef APVLV_WITH_TXT
 	&& g_ascii_strncasecmp (basename + strlen (basename) - 4,
 				".txt", 4) != 0
-#endif
-#ifdef APVLV_WITH_UMD
-	&& g_ascii_strncasecmp (basename + strlen (basename) - 4,
-				".umd", 4) != 0
-#endif
-#ifdef APVLV_WITH_HTML
-	&& g_ascii_strncasecmp (basename + strlen (basename) - 4,
-				".htm", 4) != 0
-	&& g_ascii_strncasecmp (basename + strlen (basename) - 4,
-				".html", 5) != 0
 #endif
 	)
       {
@@ -970,7 +976,7 @@ namespace apvlv
     if (dir->mType == CORE_CONTENT
 	&& ev == G_FILE_MONITOR_EVENT_CHANGED)
       {
-	gView->errormessage ("Contents is modified, apvlv reload it automatically");
+	dir->mView->errormessage ("Contents is modified, apvlv reload it automatically");
 	dir->reload ();
       }
     else if (dir->mType == CORE_DIR
@@ -1038,6 +1044,12 @@ namespace apvlv
 	      }
 	    else if (g_ascii_strncasecmp (name + strlen (name) - 4, ".pdf", 4)
 		     == 0
+		     || g_ascii_strncasecmp (name + strlen (name) - 4,
+					     ".htm", 4) == 0
+		     || g_ascii_strncasecmp (name + strlen (name) - 5,
+					     ".html", 5) == 0
+		     || g_ascii_strncasecmp (name + strlen (name) - 5,
+					     ".epub", 5) == 0
 #ifdef APVLV_WITH_DJVU
 		     || g_ascii_strncasecmp (name + strlen (name) - 5,
 					     ".djvu", 5) == 0
@@ -1047,16 +1059,6 @@ namespace apvlv
 #ifdef APVLV_WITH_TXT
 		     || g_ascii_strncasecmp (name + strlen (name) - 4,
 					     ".txt", 4) == 0
-#endif
-#ifdef APVLV_WITH_UMD
-		     || g_ascii_strncasecmp (name + strlen (name) - 4,
-					     ".umd", 4) == 0
-#endif
-#ifdef APVLV_WITH_HTML
-		     || g_ascii_strncasecmp (name + strlen (name) - 4,
-					     ".htm", 4) == 0
-		     || g_ascii_strncasecmp (name + strlen (name) - 4,
-					     ".html", 5) == 0
 #endif
 		     )
 	      {
