@@ -34,6 +34,7 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <webkit2/webkit2.h>
 
 #include <cstdlib>
 #include <cmath>
@@ -47,7 +48,21 @@ namespace apvlv
   static GtkPrintSettings *settings = NULL;
   const int APVLV_DOC_CURSOR_WIDTH = 2;
 
-  ApvlvDoc::ApvlvDoc (int w, int h, const char *zm, bool cache)
+  DISPLAY_TYPE get_display_type_by_filename (const char *name)
+  {
+    DISPLAY_TYPE type = DISPLAY_TYPE_IMAGE;
+
+    if (g_ascii_strcasecmp (name + strlen (name) - 4, ".htm") == 0
+        || g_ascii_strcasecmp (name + strlen (name) - 5, ".html") == 0
+        || g_ascii_strcasecmp (name + strlen (name) - 5, ".epub") == 0)
+      {
+        type = DISPLAY_TYPE_HTML;
+      }
+
+    return type;
+  }
+
+  ApvlvDoc::ApvlvDoc (ApvlvView *view, DISPLAY_TYPE type, int w, int h, const char *zm, bool cache): ApvlvCore(view)
   {
     mCurrentCache1 = mCurrentCache2 = NULL;
 
@@ -56,8 +71,12 @@ namespace apvlv
     mAdjInchg = false;
 
     mAutoScrollPage = gParams->valueb ("autoscrollpage");
-    mAutoScrollDoc = gParams->valueb ("autoscrolldoc");;
-    mContinuous = gParams->valueb ("continuous");;
+    mAutoScrollDoc = gParams->valueb ("autoscrolldoc");
+    mContinuous = gParams->valueb ("continuous");
+    if (type != 0)
+      {
+        mContinuous = false;
+      }
 
     mZoominit = false;
 
@@ -98,14 +117,33 @@ namespace apvlv
 		      G_CALLBACK (apvlv_doc_button_event), this);
     g_signal_connect (G_OBJECT (ebox), "motion-notify-event",
 		      G_CALLBACK (apvlv_doc_motion_event), this);
+#if GTK_CHECK_VERSION(3, 0, 0)
     gtk_container_add (GTK_CONTAINER (mScrollwin), ebox);
+#else
+    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (mScrollwin), ebox);
+#endif
 
-    mImage1 = gtk_image_new ();
-    gtk_box_pack_start (GTK_BOX (vbox), mImage1, TRUE, TRUE, 0);
-    if (mAutoScrollPage && mContinuous)
+    mDisplayType = type;
+    if (type == 0)
       {
-	mImage2 = gtk_image_new ();
-	gtk_box_pack_start (GTK_BOX (vbox), mImage2, TRUE, TRUE, 0);
+        mImg1 = gtk_image_new ();
+        gtk_box_pack_start (GTK_BOX (vbox), mImg1, TRUE, TRUE, 0);
+        if (mAutoScrollPage && mContinuous)
+          {
+            mImg2 = gtk_image_new ();
+            gtk_box_pack_start (GTK_BOX (vbox), mImg2, TRUE, TRUE, 0);
+          }
+      }
+    else if (type == 1)
+      {
+        mWeb1 = webkit_web_view_new ();
+        gtk_box_pack_start (GTK_BOX (vbox), mWeb1, TRUE, TRUE, 0);
+        g_signal_connect (mWeb1, "resource-load-started",
+                          G_CALLBACK (webview_resource_load_started_cb), this);
+        g_signal_connect (mWeb1, "load-changed",
+                          G_CALLBACK (webview_load_changed_cb), this);
+        g_signal_connect (mWeb1, "context-menu",
+                          G_CALLBACK (webview_context_menu_cb), this);
       }
 
     g_signal_connect (G_OBJECT (mVaj), "value-changed",
@@ -253,6 +291,11 @@ namespace apvlv
   {
     //    debug ("bx: %d, by: %d", bx, by);
 
+    if (mDisplayType != 0)
+      {
+        return;
+      }
+
     ApvlvDocCache *cache = mCurrentCache1;
     gint rate = cache->getheight () / mLines;
 
@@ -388,18 +431,18 @@ namespace apvlv
 	  {
 	    mCurrentCache2->getdata (true);
 	    GdkPixbuf *p = mCurrentCache2->getbuf (true);
-	    gtk_image_set_from_pixbuf (GTK_IMAGE (mImage2), p);
+	    gtk_image_set_from_pixbuf (GTK_IMAGE (mImg2), p);
 	  }
 
-	gtk_image_set_from_pixbuf (GTK_IMAGE (mImage1), pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (mImg1), pixbuf);
       }
     else
       {
 	mCurrentCache1->getdata (true);
 	GdkPixbuf *p = mCurrentCache1->getbuf (true);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (mImage1), p);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (mImg1), p);
 
-	gtk_image_set_from_pixbuf (GTK_IMAGE (mImage2), pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (mImg2), pixbuf);
       }
   }
 
@@ -557,7 +600,7 @@ namespace apvlv
       case ':':
       case '/':
       case '?':
-	gView->promptcommand (key);
+	mView->promptcommand (key);
 	return NEED_MORE;
       case 'H':
 	scrollto (0.0);
@@ -610,25 +653,29 @@ namespace apvlv
 	returnlink (ct);
 	break;
       case 't':
-	gView->newtab (helppdf.c_str ());
-	gView->open ();
+	mView->newtab (helppdf.c_str ());
+	mView->open ();
 	break;
       case 'T':
-	gView->newtab (helppdf.c_str ());
-	gView->opendir ();
+	mView->newtab (helppdf.c_str ());
+	mView->opendir ();
 	break;
       case 'o':
-	gView->open ();
+	mView->open ();
 	break;
       case 'O':
-	gView->opendir ();
+	mView->opendir ();
 	break;
       case 'r':
 	rotate (ct);
 	break;
       case 'G':
 	markposition ('\'');
-	showpage (mFile->pagesum () - 1);
+	if (!has) {
+          showpage (mFile->pagesum () - 1);
+	} else {
+          showpage (ct - 1);
+	}
 	break;
       case 'm':
       case '\'':
@@ -690,7 +737,7 @@ namespace apvlv
   {
     char rate[16];
     g_snprintf (rate, sizeof rate, "%f", mZoomrate);
-    ApvlvDoc *ndoc = new ApvlvDoc (mWidth, mHeight, rate, usecache ());
+    ApvlvDoc *ndoc = new ApvlvDoc (mView, mDisplayType, mWidth, mHeight, rate, usecache ());
     ndoc->loadfile (mFilestr, false);
     ndoc->showpage (mPagenum, scrollrate ());
     return ndoc;
@@ -762,27 +809,15 @@ namespace apvlv
 	return false;
       }
 
-    if (gParams->valueb ("noinfo"))
-      {
-	showpage (0);
-	return false;
-      }
-
     bool ret = false;
 
     infofile *fp = gInfo->file (filename);
 
-
     // correctly check
     mScrollvalue = fp->rate;
     showpage (fp->page);
     setskip (fp->skip);
 
-    // Warning
-    // I can't think a better way to scroll correctly when
-    // the page is not be displayed correctly
-    g_timeout_add (50, apvlv_doc_first_scroll_cb, this);
-
     return ret;
   }
 
@@ -825,8 +860,8 @@ namespace apvlv
   {
     if (use)
       {
-	gView->errormessage ("No pthread, can't support cache!!!");
-	gView->infomessage
+	mView->errormessage ("No pthread, can't support cache!!!");
+	mView->infomessage
 	  ("If you really have pthread, please recomplie the apvlv and try again.");
       }
   }
@@ -850,7 +885,7 @@ namespace apvlv
 
     mFile = ApvlvFile::newfile (filename);
 
-    debug ("mFile = %p", mFile);
+    //debug ("mFile = %p", mFile);
     if (mFile != NULL)
       {
 	mFilestr = filename;
@@ -863,7 +898,7 @@ namespace apvlv
 	    mAutoScrollPage = false;
 	  }
 
-	debug ("pagesum () = %d", mFile->pagesum ());
+	//debug ("pagesum () = %d", mFile->pagesum ());
 
 	if (mCurrentCache1 != NULL)
 	  {
@@ -885,7 +920,7 @@ namespace apvlv
 
 	loadlastposition (filename);
 
-	mStatus->show ();
+	mStatus->show (mContinuous);
 
 	setactive (true);
 
@@ -902,8 +937,25 @@ namespace apvlv
 
 	if (gParams->valuei ("autoreload") > 0)
 	  {
-	    mGFile = g_file_new_for_path (filename);
-	    if (mGFile)
+            if (g_file_test (filename, G_FILE_TEST_IS_SYMLINK))
+              {
+                gchar *realname = g_file_read_link (filename, NULL);
+                if (realname)
+                  {
+                    mGFile = g_file_new_for_path (realname);
+                    g_free (realname);
+                  }
+                else
+                  {
+                    mGFile = NULL;
+                  }
+              }
+            else
+              {
+                mGFile = g_file_new_for_path (filename);
+              }
+
+            if (mGFile)
 	      {
 		GError *error = NULL;
 		mGMonitor = g_file_monitor_file (mGFile, G_FILE_MONITOR_NONE, NULL, &error);
@@ -980,7 +1032,7 @@ namespace apvlv
     if (rp < 0)
       return;
 
-    debug ("show page: %d", rp);
+    //debug ("show page: %d", rp);
     mAdjInchg = true;
 
     if (mAutoScrollPage && mContinuous && !mAutoScrollDoc)
@@ -1028,19 +1080,50 @@ namespace apvlv
     if (mFile == NULL)
       return;
 
-    mCurrentCache1->set (mPagenum, mZoomrate, mRotatevalue, false);
-    GdkPixbuf *buf = mCurrentCache1->getbuf (true);
-    gtk_image_set_from_pixbuf (GTK_IMAGE (mImage1), buf);
-
-    if (mAutoScrollPage && mContinuous)
+    if (mDisplayType == 0)
       {
-	mCurrentCache2->set (convertindex (mPagenum + 1), mZoomrate,
-			     mRotatevalue, false);
-	buf = mCurrentCache2->getbuf (true);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (mImage2), buf);
+        mCurrentCache1->set (mPagenum, mZoomrate, mRotatevalue, false);
+        GdkPixbuf *buf = mCurrentCache1->getbuf (true);
+        gtk_image_set_from_pixbuf (GTK_IMAGE (mImg1), buf);
+
+        if (mAutoScrollPage && mContinuous)
+          {
+            mCurrentCache2->set (convertindex (mPagenum + 1), mZoomrate,
+                                 mRotatevalue, false);
+            buf = mCurrentCache2->getbuf (true);
+            gtk_image_set_from_pixbuf (GTK_IMAGE (mImg2), buf);
+          }
+      }
+    else if (mDisplayType == 1)
+      {
+        mFile->renderweb (mPagenum, 0, 0, mZoomrate, mRotatevalue, mWeb1);
+      }
+    mStatus->show (mContinuous);
+  }
+
+  void ApvlvDoc::srtranslate(int &rtimes, double &sr, bool single2continuous)
+  {
+    gdouble winv = gtk_adjustment_get_upper(mVaj) - gtk_adjustment_get_lower(mVaj),
+      pagewidth = gtk_adjustment_get_page_size(mVaj),
+      maxv = winv - pagewidth,
+      maxv2 = (winv - 2 * pagewidth) / 2,
+      value;
+
+    if (single2continuous)
+      {
+        value = 0.5 * pagewidth + sr * maxv2;
+        sr = (value - 0.5 * pagewidth) / maxv;
+      } else {
+      value = 0.5 * pagewidth + sr * maxv;
+      if (value > 0.5 * winv) {
+        rtimes += 1;
+        value -= 0.5 * winv;
       }
 
-    mStatus->show ();
+      if (value > 0.5 * (winv - pagewidth)) sr = 1;
+      else if (value > 0.5 * pagewidth) sr = (value - 0.5 * pagewidth) / maxv2;
+      else sr = 0;
+    }
   }
 
   void ApvlvDoc::halfnextpage (int times)
@@ -1048,19 +1131,23 @@ namespace apvlv
     double sr = scrollrate ();
     int rtimes = times / 2;
 
+    if (mAutoScrollPage && mContinuous) srtranslate(rtimes, sr, false);
+
     if (times % 2 != 0)
       {
-	if (sr > 0.5)
-	  {
-	    sr = 0;
-	    rtimes += 1;
-	  }
-	else
-	  {
-	    sr = 1;
-	  }
+        if (sr > 0.5)
+          {
+            sr = 0;
+            rtimes += 1;
+          }
+        else
+          {
+            sr = 1;
+          }
       }
 
+    if (mAutoScrollPage && mContinuous) srtranslate(rtimes, sr, true);
+
     showpage (mPagenum + rtimes, sr);
   }
 
@@ -1069,6 +1156,8 @@ namespace apvlv
     double sr = scrollrate ();
     int rtimes = times / 2;
 
+    if (mAutoScrollPage && mContinuous) srtranslate(rtimes, sr, false);
+
     if (times % 2 != 0)
       {
 	if (sr < 0.5)
@@ -1082,6 +1171,8 @@ namespace apvlv
 	  }
       }
 
+    if (mAutoScrollPage && mContinuous) srtranslate(rtimes, sr, true);
+
     showpage (mPagenum - rtimes, sr);
   }
 
@@ -1144,7 +1235,7 @@ namespace apvlv
 			     mCurrentCache1->getheight (), mZoomrate,
 			     mRotatevalue, pixbuf, (char *) pagedata,
 			     mSearchSelect, mSearchResults);
-    gtk_image_set_from_pixbuf (GTK_IMAGE (mImage1), pixbuf);
+    gtk_image_set_from_pixbuf (GTK_IMAGE (mImg1), pixbuf);
     debug ("helight num: %d", mPagenum);
   }
 
@@ -1388,7 +1479,7 @@ namespace apvlv
 	  }
 	else
 	  {
-	    gView->errormessage ("can't find word: '%s'",
+	    mView->errormessage ("can't find word: '%s'",
 				 mSearchStr.c_str ());
 	    return false;
 	  }
@@ -1425,7 +1516,7 @@ namespace apvlv
 
     if (ct % 90 != 0)
       {
-	gView->errormessage ("Not a 90 times value: %d", ct);
+	mView->errormessage ("Not a 90 times value: %d", ct);
 	return false;
       }
 
@@ -1533,7 +1624,7 @@ namespace apvlv
       }
     int r =
       gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-			       GTK_WINDOW (gView->widget ()), NULL);
+			       GTK_WINDOW (mView->widget ()), NULL);
     if (r == GTK_PRINT_OPERATION_RESULT_APPLY)
       {
 	if (settings != NULL)
@@ -1566,19 +1657,6 @@ namespace apvlv
       }
   }
 
-  gboolean ApvlvDoc::apvlv_doc_first_scroll_cb (gpointer data)
-  {
-    ApvlvDoc *doc = (ApvlvDoc *) data;
-    gboolean retry = doc->scrollto (doc->mScrollvalue) == TRUE ? FALSE : TRUE;
-    if (!retry)
-      {
-	doc->blank (0,
-		    doc->mScrollvalue * (gtk_adjustment_get_upper(doc->mVaj) - gtk_adjustment_get_lower(doc->mVaj) -
-					 gtk_adjustment_get_page_size(doc->mVaj)));
-      }
-    return retry;
-  }
-
   gboolean ApvlvDoc::apvlv_doc_first_copy_cb (gpointer data)
   {
     ApvlvDoc *doc = (ApvlvDoc *) data;
@@ -1625,7 +1703,7 @@ namespace apvlv
 
     if (ev == G_FILE_MONITOR_EVENT_CHANGED)
       {
-	gView->errormessage ("Contents is modified, apvlv reload it automatically");
+	doc->mView->errormessage ("Contents is modified, apvlv reload it automatically");
 	doc->reload ();
       }
   }
@@ -1679,432 +1757,475 @@ namespace apvlv
 	gtk_widget_show (item);
 
 	g_signal_connect (item, "activate",
-          G_CALLBACK (apvlv_doc_copytoclipboard_cb), doc);
+                          G_CALLBACK (apvlv_doc_copytoclipboard_cb), doc);
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, 0);
-      }
+#if GTK_CHECK_VERSION(3, 22, 0)
+	gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
+#else
+        gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, 0);
+#endif
       }
+  }
 
-        void
-          ApvlvDoc::apvlv_doc_copytoclipboard_cb (GtkMenuItem * item,
-          ApvlvDoc * doc)
-        {
-        doc->yank (1);
-        doc->mInVisual = VISUAL_NONE;
-        doc->blank (doc->mCurx, doc->mCury);
-      }
+  void
+  ApvlvDoc::apvlv_doc_copytoclipboard_cb (GtkMenuItem * item,
+                                          ApvlvDoc * doc)
+  {
+    doc->yank (1);
+    doc->mInVisual = VISUAL_NONE;
+    doc->blank (doc->mCurx, doc->mCury);
+  }
 
-        void
-          ApvlvDoc::apvlv_doc_motion_event (GtkWidget * box,
-          GdkEventMotion * motion, ApvlvDoc * doc)
-        {
-        doc->mInVisual = VISUAL_V;
-        double rx, ry;
-        doc->eventpos (motion->x, motion->y, &rx, &ry);
-        doc->blank (rx, ry);
-      }
+  void
+  ApvlvDoc::apvlv_doc_motion_event (GtkWidget * box,
+                                    GdkEventMotion * motion, ApvlvDoc * doc)
+  {
+    doc->mInVisual = VISUAL_V;
+    double rx, ry;
+    doc->eventpos (motion->x, motion->y, &rx, &ry);
+    doc->blank (rx, ry);
+  }
 
-        void ApvlvDoc::eventpos (double x, double y, double *rx, double *ry)
-        {
-        int dw, dh;
-        if (rx != NULL)
+  void
+  ApvlvDoc::webview_resource_load_started_cb (WebKitWebView *web_view,
+                                              WebKitWebResource *resource,
+                                              WebKitURIRequest *request,
+                                              ApvlvDoc *doc)
+  {
+    debug("resource: %s, request: %s", webkit_web_resource_get_uri (resource), webkit_uri_request_get_uri (request));
+  }
+
+  void
+  ApvlvDoc::webview_load_changed_cb (WebKitWebView *web_view,
+                                     WebKitLoadEvent event,
+                                     ApvlvDoc *doc)
+  {
+    if (event == WEBKIT_LOAD_FINISHED)
+      {
+        string anchor = doc->mFile->get_anchor();
+        if (anchor != "")
           {
-	GtkAllocation allocation;
-	gtk_widget_get_allocation(mScrollwin, &allocation);
-	dw = mPagex * mZoomrate - allocation.width;
-	dw = dw >> 1;
-	if (dw >= 0)
-	  {
-        dw = 0;
-      }
-	*rx = x + dw;
+            gchar *javasrc = g_strdup_printf("document.getElementById('%s').scrollIntoView();",
+                                             anchor.c_str());
+            webkit_web_view_run_javascript (web_view,
+                                            javasrc,
+                                            NULL, NULL, doc);
+          }
       }
+  }
 
-        if (ry != NULL)
+  gboolean
+  ApvlvDoc::webview_context_menu_cb (WebKitWebView       *web_view,
+                                     WebKitContextMenu   *context_menu,
+                                     GdkEvent            *event,
+                                     WebKitHitTestResult *hit_test_result,
+                                     ApvlvDoc *doc)
+  {
+    return TRUE;
+  }
+
+  void
+  ApvlvDoc::eventpos (double x, double y, double *rx, double *ry)
+  {
+    int dw, dh;
+    if (rx != NULL)
+      {
+        GtkAllocation allocation;
+        gtk_widget_get_allocation(mScrollwin, &allocation);
+        dw = mPagex * mZoomrate - allocation.width;
+        dw = dw >> 1;
+        if (dw >= 0)
           {
-	GtkAllocation allocation;
-	gtk_widget_get_allocation(mScrollwin, &allocation);
-	dh = mPagey * mZoomrate - allocation.height;
-	dh = dh >> 1;
-	if (dh >= 0)
-	  {
-        dh = 0;
-      }
-	*ry = y + dh;
-      }
+            dw = 0;
+          }
+        *rx = x + dw;
       }
 
-        ApvlvDocCache::ApvlvDocCache (ApvlvFile * file)
-        {
-        mFile = file;
-        mPagenum = -1;
+    if (ry != NULL)
+      {
+        GtkAllocation allocation;
+        gtk_widget_get_allocation(mScrollwin, &allocation);
+        dh = mPagey * mZoomrate - allocation.height;
+        dh = dh >> 1;
+        if (dh >= 0)
+          {
+            dh = 0;
+          }
+        *ry = y + dh;
+      }
+  }
+
+  ApvlvDocCache::ApvlvDocCache (ApvlvFile * file)
+  {
+    mFile = file;
+    mPagenum = -1;
+    mLines = NULL;
+    mData = NULL;
+    mSize = 0;
+    mBuf = NULL;
+    mLinks = NULL;
+  }
+
+  void ApvlvDocCache::set (guint p, double zm, guint rot, bool delay)
+  {
+    mPagenum = p;
+    mZoom = zm;
+    mRotate = rot;
+
+    if (mLines != NULL)
+      {
+        delete mLines;
         mLines = NULL;
+      }
+
+    if (mData != NULL)
+      {
+        delete[]mData;
         mData = NULL;
-        mSize = 0;
+      }
+    if (mBuf != NULL)
+      {
+        g_object_unref (mBuf);
         mBuf = NULL;
+      }
+    if (mLinks != NULL)
+      {
+        delete mLinks;
         mLinks = NULL;
       }
+    mInverted = gParams->valueb ("inverted");
 
-        void ApvlvDocCache::set (guint p, double zm, guint rot, bool delay)
-        {
-        mPagenum = p;
-        mZoom = zm;
-        mRotate = rot;
+    load (this);
+  }
 
-        if (mLines != NULL)
+  void ApvlvDocCache::load (ApvlvDocCache * ac)
+  {
+    int c = ac->mFile->pagesum ();
+
+    if (ac->mPagenum < 0 || ac->mPagenum >= c)
+      {
+        debug ("no this page: %d", ac->mPagenum);
+        return;
+      }
+
+    double tpagex, tpagey;
+    ac->mFile->pagesize (ac->mPagenum, ac->mRotate, &tpagex, &tpagey);
+
+    ac->mWidth = MAX ((tpagex * ac->mZoom + 0.5), 1);
+    ac->mHeight = MAX ((tpagey * ac->mZoom + 0.5), 1);
+
+    // this is very import to get the double times size data
+    // the 2ed chunk data will be for output
+    ac->mSize = ac->mWidth * ac->mHeight * 3;
+    guchar *dat = new guchar[2 * ac->mSize];
+
+    GdkPixbuf *bu = gdk_pixbuf_new_from_data (dat, GDK_COLORSPACE_RGB,
+                                              FALSE,
+                                              8,
+                                              ac->mWidth, ac->mHeight,
+                                              3 * ac->mWidth,
+                                              NULL, NULL);
+    //debug ("ac->mFile: %p", ac->mFile);
+    ac->mFile->render (ac->mPagenum, ac->mWidth, ac->mHeight, ac->mZoom,
+                       ac->mRotate, bu, (char *) dat);
+    if (ac->mInverted)
+      {
+        invert_pixbuf (bu);
+      }
+    // backup the pixbuf data
+    memcpy (dat + ac->mSize, dat, ac->mSize);
+
+    if (ac->mLinks)
+      {
+        delete ac->mLinks;
+      }
+    ac->mLinks = ac->mFile->getlinks (ac->mPagenum);
+    //debug ("has mLinkMappings: %p", ac->mLinks);
+
+    ac->mData = dat;
+    ac->mBuf = bu;
+
+    ac->preparelines (0, 0, tpagex, tpagey);
+  }
+
+  ApvlvDocCache::~ApvlvDocCache ()
+  {
+    if (mLinks)
+      {
+        delete mLinks;
+      }
+
+    if (mLines != NULL)
+      {
+        delete mLines;
+      }
+
+    if (mData != NULL)
+      {
+        delete[]mData;
+      }
+
+    if (mBuf != NULL)
+      {
+        g_object_unref (mBuf);
+      }
+  }
+
+  guint ApvlvDocCache::getpagenum ()
+  {
+    return mPagenum;
+  }
+
+  /*
+   * get the cache data
+   * @param: wait, if not wait, not wait the buffer be prepared
+   * @return: the buffer
+   * */
+  guchar *ApvlvDocCache::getdata (bool wait)
+  {
+    memcpy (mData, mData + mSize, mSize);
+    return mData;
+  }
+
+  /*
+   * get the cache GdkPixbuf
+   * @param: wait, if not wait, not wait the pixbuf be prepared
+   * @return: the buffer
+   * */
+  GdkPixbuf *ApvlvDocCache::getbuf (bool wait)
+  {
+    return mBuf;
+  }
+
+  double ApvlvDocCache::getwidth ()
+  {
+    return mWidth;
+  }
+
+  double ApvlvDocCache::getheight ()
+  {
+    return mHeight;
+  }
+
+  ApvlvLinks *ApvlvDocCache::getlinks ()
+  {
+    return mLinks;
+  }
+
+  bool ApvlvDocCache::canselect ()
+  {
+    return mLines != NULL ? true : false;
+  }
+
+  ApvlvWord *ApvlvDocCache::getword (int x, int y)
+  {
+    ApvlvLine *line;
+
+    line = getline (x, y);
+    if (line != NULL)
+      {
+        vector < ApvlvWord >::iterator itr;
+        for (itr = line->mWords.begin (); itr != line->mWords.end (); itr++)
           {
-	delete mLines;
-	mLines = NULL;
+            debug ("itr: %f,%f", itr->pos.y1, itr->pos.y2);
+            if (x >= itr->pos.x1 && x <= itr->pos.x2)
+              {
+                return &(*itr);
+              }
+          }
+      }
+    return NULL;
+  }
+
+  ApvlvLine *ApvlvDocCache::getline (double x, double y)
+  {
+    debug ("getline: %f", y);
+
+    if (mLines == NULL)
+      {
+        return NULL;
       }
 
-        if (mData != NULL)
-          {
-	delete[]mData;
-	mData = NULL;
-      }
-        if (mBuf != NULL)
-          {
-	g_object_unref (mBuf);
-	mBuf = NULL;
-      }
-        if (mLinks != NULL)
-          {
-	delete mLinks;
-	mLinks = NULL;
-      }
-        mInverted = gParams->valueb ("inverted");
-
-        load (this);
-      }
-
-        void ApvlvDocCache::load (ApvlvDocCache * ac)
-        {
-        int c = ac->mFile->pagesum ();
-
-        if (ac->mPagenum < 0 || ac->mPagenum >= c)
-          {
-	debug ("no this page: %d", ac->mPagenum);
-	return;
-      }
-
-        double tpagex, tpagey;
-        ac->mFile->pagesize (ac->mPagenum, ac->mRotate, &tpagex, &tpagey);
-
-        ac->mWidth = MAX ((tpagex * ac->mZoom + 0.5), 1);
-        ac->mHeight = MAX ((tpagey * ac->mZoom + 0.5), 1);
-
-        // this is very import to get the double times size data
-        // the 2ed chunk data will be for output
-        ac->mSize = ac->mWidth * ac->mHeight * 3;
-        guchar *dat = new guchar[2 * ac->mSize];
-
-        GdkPixbuf *bu = gdk_pixbuf_new_from_data (dat, GDK_COLORSPACE_RGB,
-          FALSE,
-          8,
-          ac->mWidth, ac->mHeight,
-          3 * ac->mWidth,
-          NULL, NULL);
-        debug ("ac->mFile: %p", ac->mFile);
-        ac->mFile->render (ac->mPagenum, ac->mWidth, ac->mHeight, ac->mZoom,
-          ac->mRotate, bu, (char *) dat);
-        if (ac->mInverted)
-          {
-	invert_pixbuf (bu);
-      }
-        // backup the pixbuf data
-        memcpy (dat + ac->mSize, dat, ac->mSize);
-
-        if (ac->mLinks)
-          {
-	delete ac->mLinks;
-      }
-        ac->mLinks = ac->mFile->getlinks (ac->mPagenum);
-        debug ("has mLinkMappings: %p", ac->mLinks);
-
-        ac->mData = dat;
-        ac->mBuf = bu;
-
-        ac->preparelines (0, 0, tpagex, tpagey);
-      }
-
-        ApvlvDocCache::~ApvlvDocCache ()
-        {
-        if (mLinks)
-          {
-	delete mLinks;
-      }
-
-        if (mLines != NULL)
-          {
-	delete mLines;
-      }
-
-        if (mData != NULL)
-          {
-	delete[]mData;
-      }
-
-        if (mBuf != NULL)
-          {
-	g_object_unref (mBuf);
-      }
-      }
-
-        guint ApvlvDocCache::getpagenum ()
-        {
-        return mPagenum;
-      }
-
-        /*
-         * get the cache data
-         * @param: wait, if not wait, not wait the buffer be prepared
-         * @return: the buffer
-         * */
-        guchar *ApvlvDocCache::getdata (bool wait)
-        {
-        memcpy (mData, mData + mSize, mSize);
-        return mData;
-      }
-
-        /*
-         * get the cache GdkPixbuf
-         * @param: wait, if not wait, not wait the pixbuf be prepared
-         * @return: the buffer
-         * */
-        GdkPixbuf *ApvlvDocCache::getbuf (bool wait)
-        {
-        return mBuf;
-      }
-
-        double ApvlvDocCache::getwidth ()
-        {
-        return mWidth;
-      }
-
-        double ApvlvDocCache::getheight ()
-        {
-        return mHeight;
-      }
-
-        ApvlvLinks *ApvlvDocCache::getlinks ()
-        {
-        return mLinks;
-      }
-
-        bool ApvlvDocCache::canselect ()
-        {
-        return mLines != NULL ? true : false;
-      }
-
-        ApvlvWord *ApvlvDocCache::getword (int x, int y)
-        {
-        ApvlvLine *line;
-
-        line = getline (x, y);
-        if (line != NULL)
-          {
-	vector < ApvlvWord >::iterator itr;
-	for (itr = line->mWords.begin (); itr != line->mWords.end (); itr++)
-	  {
+    vector < ApvlvLine >::iterator itr;
+    for (itr = mLines->begin (); itr != mLines->end (); itr++)
+      {
         debug ("itr: %f,%f", itr->pos.y1, itr->pos.y2);
-        if (x >= itr->pos.x1 && x <= itr->pos.x2)
+        if (y >= itr->pos.y2 && y <= itr->pos.y1)
           {
-        return &(*itr);
+            return &(*itr);
+          }
       }
-      }
-      }
-        return NULL;
+    return NULL;
+  }
+
+  void ApvlvDocCache::preparelines (double x1, double y1, double x2,
+                                    double y2)
+  {
+    if (strcmp (gParams->values ("doubleclick"), "page") == 0
+        || strcmp (gParams->values ("doubleclick"), "none") == 0)
+      {
+        return;
       }
 
-        ApvlvLine *ApvlvDocCache::getline (double x, double y)
-        {
-        debug ("getline: %f", y);
+    gchar *content = NULL;
+    mFile->pagetext (mPagenum, x1, y1, x2, y2, &content);
+    if (content != NULL)
+      {
+        ApvlvPoses *results;
+        string word;
 
-        if (mLines == NULL)
-          {
-	return NULL;
-      }
+        mLines = new vector < ApvlvLine >;
 
-        vector < ApvlvLine >::iterator itr;
-        for (itr = mLines->begin (); itr != mLines->end (); itr++)
-          {
-	debug ("itr: %f,%f", itr->pos.y1, itr->pos.y2);
-	if (y >= itr->pos.y2 && y <= itr->pos.y1)
-	  {
-        return &(*itr);
-      }
-      }
-        return NULL;
-      }
+        ApvlvPos lastpos = { 0, 0, 0, 0 };
 
-        void ApvlvDocCache::preparelines (double x1, double y1, double x2,
-          double y2)
-        {
-        if (strcmp (gParams->values ("doubleclick"), "page") == 0
-          || strcmp (gParams->values ("doubleclick"), "none") == 0)
+        if (strcmp (gParams->values ("doubleclick"), "word") == 0)
           {
-	return;
-      }
-
-        gchar *content = NULL;
-        mFile->pagetext (mPagenum, x1, y1, x2, y2, &content);
-        if (content != NULL)
-          {
-	ApvlvPoses *results;
-	string word;
-
-	mLines = new vector < ApvlvLine >;
-
-	ApvlvPos lastpos = { 0, 0, 0, 0 };
-
-	if (strcmp (gParams->values ("doubleclick"), "word") == 0)
-	  {
-        stringstream ss (content);
-        while (!ss.eof ())
-          {
-        ss >> word;
-        results = mFile->pagesearch (mPagenum, word.c_str ());
-        if (results != NULL)
-          {
-        lastpos = prepare_add (lastpos, results, word.c_str ());
-        delete results;
-      }
-      }
-      }
-	else
-	  {
-        gchar **v, *p;
-        int i;
-
-        v = g_strsplit (content, "\n", -1);
-        if (v != NULL)
-          {
-        for (i = 0; v[i] != NULL; ++i)
-          {
-        p = v[i];
-        while (*p != '\0' && isspace (*p))
-          {
-        p++;
-      }
-        if (*p == '\0')
-          {
-        continue;
-      }
-        debug ("search [%s]", p);
-        results = mFile->pagesearch (mPagenum, p);
-        if (results != NULL)
-          {
-        lastpos = prepare_add (lastpos, results, p);
-        delete results;
-      }
-      }
-        g_strfreev (v);
-      }
-      }
-
-	vector < ApvlvLine >::iterator it;
-	for (it = mLines->begin (); it != mLines->end (); it++)
-	  {
-        //              debug ("line: %f, %f, %f, %f", it->pos.x1, it->pos.y1,
-        //                     it->pos.x2, it->pos.y2);
-        vector < ApvlvWord >::iterator wit;
-        for (wit = it->mWords.begin (); wit != it->mWords.end (); wit++)
-          {
-        //                 debug ("word: %f, %f, %f, %f: %s", wit->pos.x1, wit->pos.y1,
-        //                         wit->pos.x2, wit->pos.y2, wit->word.c_str ());
-      }
-      }
-	g_free (content);
-      }
-      }
-
-        ApvlvPos ApvlvDocCache::prepare_add (ApvlvPos & lastpos,
-          ApvlvPoses * results, const char *word)
-        {
-        ApvlvPoses::iterator itr;
-        for (itr = results->begin (); itr != results->end (); itr++)
-          {
-	itr->x1 *= mZoom;
-	itr->x2 *= mZoom;
-	itr->y1 = mHeight - itr->y1 * mZoom;
-	itr->y2 = mHeight - itr->y2 * mZoom;
-
-	if ((lastpos.y2 < itr->y2)
-          || (lastpos.y2 - itr->y2 < 0.0001 && lastpos.x2 < itr->x2))
-	  {
-        debug ("[%s] x1:%f, x2:%f, y1:%f, y2:%f", word, itr->x1,
-          itr->x2, itr->y1, itr->y2);
-        break;
-      }
-      }
-
-        if (itr == results->end ())
-          {
-	itr = results->begin ();
-      }
-
-        //      debug ("itr: %f, %f, %f, %f", itr->x1, itr->y1, itr->x2, itr->y2);
-        vector < ApvlvLine >::iterator litr;
-        for (litr = mLines->begin (); litr != mLines->end (); litr++)
-          {
-	//          debug ("litr: %f, %f", litr->pos.y1, litr->pos.y2);
-	if (fabs (itr->y1 - litr->pos.y1) < 0.0001
-                                            && fabs (itr->y2 - litr->pos.y2) < 0.0001)
-	  {
-        break;
-      }
-      }
-
-        ApvlvWord aword = { *itr, word };
-        if (litr != mLines->end ())
-          {
-	litr->mWords.push_back (aword);
-	if (itr->x1 < litr->pos.x1)
-	  {
-        litr->pos.x1 = itr->x1;
-      }
-	if (itr->x2 > litr->pos.x2)
-	  {
-        litr->pos.x2 = itr->x2;
-      }
-      }
+            stringstream ss (content);
+            while (!ss.eof ())
+              {
+                ss >> word;
+                results = mFile->pagesearch (mPagenum, word.c_str ());
+                if (results != NULL)
+                  {
+                    lastpos = prepare_add (lastpos, results, word.c_str ());
+                    delete results;
+                  }
+              }
+          }
         else
           {
-	ApvlvLine line;
-	line.pos = *itr;
-	line.mWords.push_back (aword);
-	mLines->push_back (line);
-      }
+            gchar **v, *p;
+            int i;
 
-        return *itr;
-      }
+            v = g_strsplit (content, "\n", -1);
+            if (v != NULL)
+              {
+                for (i = 0; v[i] != NULL; ++i)
+                  {
+                    p = v[i];
+                    while (*p != '\0' && isspace (*p))
+                      {
+                        p++;
+                      }
+                    if (*p == '\0')
+                      {
+                        continue;
+                      }
+                    debug ("search [%s]", p);
+                    results = mFile->pagesearch (mPagenum, p);
+                    if (results != NULL)
+                      {
+                        lastpos = prepare_add (lastpos, results, p);
+                        delete results;
+                      }
+                  }
+                g_strfreev (v);
+              }
+          }
 
-        ApvlvDocStatus::ApvlvDocStatus (ApvlvDoc * doc)
-        {
-        mDoc = doc;
-        for (int i = 0; i < AD_STATUS_SIZE; ++i)
+        vector < ApvlvLine >::iterator it;
+        for (it = mLines->begin (); it != mLines->end (); it++)
           {
-	mStlab[i] = gtk_label_new ("");
-	gtk_box_pack_start (GTK_BOX (mHbox), mStlab[i], FALSE, FALSE, 0);
-      }
+            //              debug ("line: %f, %f, %f, %f", it->pos.x1, it->pos.y1,
+            //                     it->pos.x2, it->pos.y2);
+            vector < ApvlvWord >::iterator wit;
+            for (wit = it->mWords.begin (); wit != it->mWords.end (); wit++)
+              {
+                //                 debug ("word: %f, %f, %f, %f: %s", wit->pos.x1, wit->pos.y1,
+                //                         wit->pos.x2, wit->pos.y2, wit->word.c_str ());
+              }
+          }
+        g_free (content);
       }
+  }
 
-        ApvlvDocStatus::~ApvlvDocStatus ()
-        {
-      }
+  ApvlvPos ApvlvDocCache::prepare_add (ApvlvPos & lastpos,
+                                       ApvlvPoses * results, const char *word)
+  {
+    ApvlvPoses::iterator itr;
+    for (itr = results->begin (); itr != results->end (); itr++)
+      {
+        itr->x1 *= mZoom;
+        itr->x2 *= mZoom;
+        itr->y1 = mHeight - itr->y1 * mZoom;
+        itr->y2 = mHeight - itr->y2 * mZoom;
 
-        void ApvlvDocStatus::active (bool act)
-        {
-        for (unsigned int i = 0; i < AD_STATUS_SIZE; ++i)
+        if ((lastpos.y2 < itr->y2)
+            || (lastpos.y2 - itr->y2 < 0.0001 && lastpos.x2 < itr->x2))
           {
+            debug ("[%s] x1:%f, x2:%f, y1:%f, y2:%f", word, itr->x1,
+                   itr->x2, itr->y1, itr->y2);
+            break;
+          }
+      }
+
+    if (itr == results->end ())
+      {
+        itr = results->begin ();
+      }
+
+    //      debug ("itr: %f, %f, %f, %f", itr->x1, itr->y1, itr->x2, itr->y2);
+    vector < ApvlvLine >::iterator litr;
+    for (litr = mLines->begin (); litr != mLines->end (); litr++)
+      {
+        //          debug ("litr: %f, %f", litr->pos.y1, litr->pos.y2);
+        if (fabs (itr->y1 - litr->pos.y1) < 0.0001
+            && fabs (itr->y2 - litr->pos.y2) < 0.0001)
+          {
+            break;
+          }
+      }
+
+    ApvlvWord aword = { *itr, word };
+    if (litr != mLines->end ())
+      {
+        litr->mWords.push_back (aword);
+        if (itr->x1 < litr->pos.x1)
+          {
+            litr->pos.x1 = itr->x1;
+          }
+        if (itr->x2 > litr->pos.x2)
+          {
+            litr->pos.x2 = itr->x2;
+          }
+      }
+    else
+      {
+        ApvlvLine line;
+        line.pos = *itr;
+        line.mWords.push_back (aword);
+        mLines->push_back (line);
+      }
+
+    return *itr;
+  }
+
+  ApvlvDocStatus::ApvlvDocStatus (ApvlvDoc * doc)
+  {
+    mDoc = doc;
+    for (int i = 0; i < AD_STATUS_SIZE; ++i)
+      {
+        mStlab[i] = gtk_label_new ("");
+        gtk_box_pack_start (GTK_BOX (mHbox), mStlab[i], FALSE, FALSE, 0);
+      }
+  }
+
+  ApvlvDocStatus::~ApvlvDocStatus ()
+  {
+  }
+
+  void ApvlvDocStatus::active (bool act)
+  {
+    for (unsigned int i = 0; i < AD_STATUS_SIZE; ++i)
+      {
 #if GTK_CHECK_VERSION(3, 0, 0)
-        gtk_widget_override_color (mStlab[i],
-                                   (act)? GTK_STATE_FLAG_ACTIVE:
-                                   GTK_STATE_FLAG_INSENSITIVE, NULL);
+        gtk_widget_set_state_flags (mStlab[i],
+                                    (act)? GTK_STATE_FLAG_ACTIVE:
+                                    GTK_STATE_FLAG_INSENSITIVE, TRUE);
 #else
-	gtk_widget_modify_fg (mStlab[i],
-			      (act) ? GTK_STATE_ACTIVE :
-			      GTK_STATE_INSENSITIVE, NULL);
+        gtk_widget_modify_fg (mStlab[i],
+                              (act) ? GTK_STATE_ACTIVE:
+                              GTK_STATE_INSENSITIVE, NULL);
 #endif
       }
   }
@@ -2118,31 +2239,34 @@ namespace apvlv
     sw[3] = sw[1] >> 1;
     for (unsigned int i = 0; i < AD_STATUS_SIZE; ++i)
       {
-	gtk_widget_set_size_request (mStlab[i], sw[i], h);
+        gtk_widget_set_size_request (mStlab[i], sw[i], h);
       }
   }
 
-  void ApvlvDocStatus::show ()
+  void ApvlvDocStatus::show (bool mContinuous)
   {
     if (mDoc->filename ())
       {
-	gint pn = mDoc->pagenumber ();
+        gint pn = mDoc->pagenumber (),
+          totpn = mDoc->file ()->pagesum ();
+        gdouble sr = mDoc->scrollrate ();
+        int tmprtimes = 0;
+        mDoc->srtranslate(tmprtimes, sr, false);
 
-	char temp[AD_STATUS_SIZE][256];
-	gchar *bn;
-	bn = g_path_get_basename (mDoc->filename ());
-	g_snprintf (temp[0], sizeof temp[0], "%s", bn);
-	g_snprintf (temp[1], sizeof temp[1], "%d/%d", pn,
-		    mDoc->file ()->pagesum ());
-	g_snprintf (temp[2], sizeof temp[2], "%d%%",
-		    (int) (mDoc->zoomvalue () * 100));
-	g_snprintf (temp[3], sizeof temp[3], "%d%%",
-		    (int) (mDoc->scrollrate () * 100));
-	for (unsigned int i = 0; i < AD_STATUS_SIZE; ++i)
-	  {
-	    gtk_label_set_text (GTK_LABEL (mStlab[i]), temp[i]);
-	  }
-	g_free (bn);
+        char temp[AD_STATUS_SIZE][256];
+        gchar *bn;
+        bn = g_path_get_basename (mDoc->filename ());
+        g_snprintf (temp[0], sizeof temp[0], "%s", bn);
+        g_snprintf (temp[1], sizeof temp[1], "%d/%d", pn, totpn);
+        g_snprintf (temp[2], sizeof temp[2], "%d%%",
+                    (int) (mDoc->zoomvalue () * 100));
+        g_snprintf (temp[3], sizeof temp[3], "%d%%",
+                    (int) ((sr + pn - 1.0) / totpn * 100));
+        for (unsigned int i = 0; i < AD_STATUS_SIZE; ++i)
+          {
+            gtk_label_set_text (GTK_LABEL (mStlab[i]), temp[i]);
+          }
+        g_free (bn);
       }
   }
 
@@ -2165,15 +2289,15 @@ namespace apvlv
     height = gdk_pixbuf_get_height (pixbuf);
     for (x = 0; x < width; x++)
       {
-	for (y = 0; y < height; y++)
-	  {
-	    /* Calculate pixel's offset into the data array. */
-	    p = data + x * n_channels + y * rowstride;
-	    /* Change the RGB values */
-	    p[0] = 255 - p[0];
-	    p[1] = 255 - p[1];
-	    p[2] = 255 - p[2];
-	  }
+        for (y = 0; y < height; y++)
+          {
+            /* Calculate pixel's offset into the data array. */
+            p = data + x * n_channels + y * rowstride;
+            /* Change the RGB values */
+            p[0] = 255 - p[0];
+            p[1] = 255 - p[1];
+            p[2] = 255 - p[2];
+          }
       }
   }
 }
