@@ -32,9 +32,8 @@
 #include "ApvlvCmds.h"
 #include "ApvlvView.h"
 
-#ifdef APVLV_WITH_HTML
-# include "ApvlvHtm.h"
-#endif
+#include "ApvlvHtm.h"
+#include "ApvlvEpub.h"
 
 #include <glib.h>
 #include <glib/gprintf.h>
@@ -50,15 +49,20 @@
 
 namespace apvlv
 {
-  ApvlvView *gView = NULL;
-
   const int ApvlvView::APVLV_MENU_HEIGHT = 30;
   const int ApvlvView::APVLV_CMD_BAR_HEIGHT = 20;
   const int ApvlvView::APVLV_TABS_HEIGHT = 36;
 
-  ApvlvView::ApvlvView (const char *filename):mCurrTabPos (-1)
+  ApvlvView::ApvlvView (ApvlvView *parent):mCurrTabPos (-1), mCmds(this)
   {
+    mParent = parent;
+    if (mParent)
+      {
+        mParent->append_child (this);
+      }
+
     mProCmd = 0;
+    mProCmdCnt = 0;
 
     mCurrHistroy = -1;
 
@@ -66,14 +70,26 @@ namespace apvlv
 
     mMainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
+    mRootWindow = NULL;
+
     int w = gParams->valuei ("width");
     int h = gParams->valuei ("height");
 
     if (gParams->valueb ("fullscreen"))
       {
-	GdkScreen *scr = gdk_screen_get_default ();
-	mWidth = gdk_screen_get_width (scr);
-	mHeight = gdk_screen_get_height (scr);
+#if GTK_CHECK_VERSION(3, 22, 0)
+        GdkRectangle rect[1];
+        GdkDisplay *display = gdk_display_get_default ();
+        GdkMonitor *monitor = gdk_display_get_primary_monitor (display);
+        gdk_monitor_get_geometry (monitor, rect);
+        mWidth = rect->width;
+	mHeight = rect->height;
+#else
+        GdkScreen *scr = gdk_screen_get_default ();
+        mWidth = gdk_screen_get_width (scr);
+        mHeight = gdk_screen_get_height (scr);
+#endif
+        debug ("get screen size: %dx%d", mWidth, mHeight);
 	fullscreen ();
       }
     else
@@ -105,22 +121,6 @@ namespace apvlv
     gtk_notebook_set_scrollable (GTK_NOTEBOOK (mTabContainer), TRUE);
     gtk_box_pack_start (GTK_BOX (mViewBox), mTabContainer, TRUE, TRUE, 0);
 
-    if (newtab (filename) == false)
-      {
-	bool ret = false;
-	gchar *rpath = absolutepath (helppdf.c_str ());
-	if (rpath)
-	  {
-	    ret = newtab (rpath);
-	    g_free (rpath);
-	  }
-
-	if (ret == false)
-	  {
-	    exit (1);
-	  }
-      }
-
     mCommandBar = gtk_entry_new ();
     gtk_box_pack_end (GTK_BOX (mViewBox), mCommandBar, FALSE, FALSE, 0);
 
@@ -148,6 +148,18 @@ namespace apvlv
 
   ApvlvView::~ApvlvView ()
   {
+    if (mParent)
+      {
+        mParent->erase_child (this);
+      }
+
+    std::vector <ApvlvView *>::const_iterator itr;
+    while (!mChildren.empty())
+      {
+        itr = mChildren.begin();
+        delete *itr;
+      }
+
     size_t i;
 
     delete mMenu;
@@ -169,7 +181,7 @@ namespace apvlv
 
   void ApvlvView::show ()
   {
-    gtk_main ();
+    gtk_widget_show_all (widget ());
   }
 
   GtkWidget *ApvlvView::widget ()
@@ -203,7 +215,7 @@ namespace apvlv
 						  NULL);
     infofile *fp = gInfo->file (0);
     dirname =
-      fp ? g_dirname (fp->file.c_str ()) :
+      fp ? g_path_get_dirname (fp->file.c_str ()) :
       g_strdup (gParams->values ("defaultdir"));
     debug ("lastfile: [%s], dirname: [%s]", fp ? fp->file.c_str () : "",
 	   dirname);
@@ -214,6 +226,14 @@ namespace apvlv
     gtk_file_filter_add_mime_type (filter, "PDF File");
     gtk_file_filter_add_pattern (filter, "*.pdf");
     gtk_file_filter_add_pattern (filter, "*.PDF");
+    gtk_file_filter_add_mime_type (filter, "HTML File");
+    gtk_file_filter_add_pattern (filter, "*.HTM");
+    gtk_file_filter_add_pattern (filter, "*.htm");
+    gtk_file_filter_add_pattern (filter, "*.HTML");
+    gtk_file_filter_add_pattern (filter, "*.html");
+    gtk_file_filter_add_mime_type (filter, "ePub File");
+    gtk_file_filter_add_pattern (filter, "*.EPUB");
+    gtk_file_filter_add_pattern (filter, "*.epub");
 #ifdef APVLV_WITH_DJVU
     gtk_file_filter_add_mime_type (filter, "DJVU File");
     gtk_file_filter_add_pattern (filter, "*.DJV");
@@ -225,18 +245,6 @@ namespace apvlv
     gtk_file_filter_add_mime_type (filter, "TXT File");
     gtk_file_filter_add_pattern (filter, "*.TXT");
     gtk_file_filter_add_pattern (filter, "*.txt");
-#endif
-#ifdef APVLV_WITH_UMD
-    gtk_file_filter_add_mime_type (filter, "UMD File");
-    gtk_file_filter_add_pattern (filter, "*.UMD");
-    gtk_file_filter_add_pattern (filter, "*.umd");
-#endif
-#ifdef APVLV_WITH_HTML
-    gtk_file_filter_add_mime_type (filter, "HTML File");
-    gtk_file_filter_add_pattern (filter, "*.HTM");
-    gtk_file_filter_add_pattern (filter, "*.htm");
-    gtk_file_filter_add_pattern (filter, "*.HTML");
-    gtk_file_filter_add_pattern (filter, "*.html");
 #endif
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dia), filter);
 
@@ -245,20 +253,7 @@ namespace apvlv
       {
 	gchar *filename =
 	  gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dia));
-
-#ifdef APVLV_WITH_HTML
-	if (g_ascii_strcasecmp (filename + strlen (filename) - 4,
-				".htm") == 0
-	    || g_ascii_strcasecmp (filename + strlen (filename) - 5,
-				   ".html") == 0)
-	  {
-	    loadhtml (filename);
-	  }
-	else
-#endif
-	  {
-	    loadfile (filename);
-	  }
+        loadfile (filename);
 	g_free (filename);
       }
     gtk_widget_destroy (dia);
@@ -277,7 +272,7 @@ namespace apvlv
 						  GTK_RESPONSE_ACCEPT,
 						  NULL);
     infofile *fp = gInfo->file (0);
-    dirname = fp ? g_dirname (fp->file.c_str ()) :
+    dirname = fp ? g_path_get_dirname (fp->file.c_str ()) :
       g_strdup (gParams->values ("defaultdir"));
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dia), dirname);
     g_free (dirname);
@@ -301,7 +296,7 @@ namespace apvlv
       {
 	int w, h;
 	currentWindow ()->getsize (&w, &h);
-	ndoc = new ApvlvDir (w, h);
+	ndoc = new ApvlvDir (this, w, h);
 	if (!ndoc->loadfile (path))
 	  {
 	    delete ndoc;
@@ -315,29 +310,6 @@ namespace apvlv
     return true;
   }
 
-#ifdef APVLV_WITH_HTML
-  bool ApvlvView::loadhtml (const char *path)
-  {
-    ApvlvCore *ndoc = hasloaded (path, CORE_DIR);
-    if (ndoc == NULL)
-      {
-	int w, h;
-	currentWindow ()->getsize (&w, &h);
-	ndoc = new ApvlvHTML (w, h);
-	if (!ndoc->loadfile (path))
-	  {
-	    delete ndoc;
-	    return false;
-	  }
-	regloaded (ndoc);
-      }
-
-    currentWindow ()->setCore (ndoc);
-    updatetabname ();
-    return true;
-  }
-#endif
-
   bool ApvlvView::loadfile (string file)
   {
     return loadfile (file.c_str ());
@@ -370,17 +342,21 @@ namespace apvlv
     mRootWindow->setsize (mWidth, adjheight ());
   }
 
-  bool ApvlvView::newtab (const char *filename)
+  bool ApvlvView::newtab (const char *filename, bool disable_content)
   {
     ApvlvCore *ndoc;
-    ndoc = hasloaded (filename,
-		      gParams->valueb ("content") ? CORE_CONTENT : CORE_DOC);
+    int core_type = gParams->valueb ("content") ? CORE_CONTENT : CORE_DOC;
+    if (disable_content)
+      {
+        core_type = CORE_DOC;
+      }
+    ndoc = hasloaded (filename, core_type);
 
     if (ndoc == NULL)
       {
-	if (gParams->valueb ("content"))
+	if (core_type == CORE_CONTENT)
 	  {
-	    ndoc = new ApvlvDir (mWidth, adjheight ());
+	    ndoc = new ApvlvDir (this, mWidth, adjheight ());
 	    if (!ndoc->loadfile (filename))
 	      {
 		delete ndoc;
@@ -390,8 +366,9 @@ namespace apvlv
 
 	if (ndoc == NULL)
 	  {
+            DISPLAY_TYPE type = get_display_type_by_filename (filename);
 	    ndoc =
-	      new ApvlvDoc (mWidth, mHeight, gParams->values ("zoom"), false);
+	      new ApvlvDoc (this, type, mWidth, mHeight, gParams->values ("zoom"), false);
 	    if (!ndoc->loadfile (filename))
 	      {
 		delete ndoc;
@@ -445,6 +422,14 @@ namespace apvlv
     return true;
   }
 
+  bool ApvlvView::newview (const gchar *filename, gint pn, bool disable_content)
+  {
+    ApvlvView *view = new ApvlvView (this);
+    view->newtab (filename, disable_content);
+    view->crtadoc()->showpage (pn);
+    return TRUE;
+  }
+
   int ApvlvView::new_tabcontext (ApvlvCore * core, bool insertAfterCurr)
   {
     ApvlvWindow *troot = new ApvlvWindow (core);
@@ -484,7 +469,7 @@ namespace apvlv
     mTabList.erase (remPos);
     if (c == (int) mTabList.size ())
       {
-	gView->errormessage ("erase failed to remove context");
+	errormessage ("erase failed to remove context");
       }
 
     if ((int) mTabList.size () <= 1)
@@ -512,6 +497,15 @@ namespace apvlv
       }
   }
 
+  void ApvlvView::back_tabcontext (int tabPos)
+  {
+    asst (tabPos >= 0 && tabPos < (int) mTabList.size ());
+
+    mCurrTabPos = tabPos;
+    mRootWindow = mTabList[tabPos].root;
+    ApvlvWindow::setcurrentWindow (NULL, mTabList[tabPos].curr);
+  }
+
   bool ApvlvView::loadfile (const char *filename)
   {
     if (filename == NULL || *filename == '\0')
@@ -538,7 +532,7 @@ namespace apvlv
 	win->getsize (&w, &h);
 	if (gParams->valueb ("content"))
 	  {
-	    ndoc = new ApvlvDir (w, h);
+	    ndoc = new ApvlvDir (this, w, h);
 	    if (!ndoc->loadfile (abpath))
 	      {
 		debug ("can't load");
@@ -549,7 +543,8 @@ namespace apvlv
 
 	if (ndoc == NULL)
 	  {
-	    ndoc = new ApvlvDoc (w, h, gParams->values ("zoom"), false);
+            DISPLAY_TYPE type = get_display_type_by_filename (filename);
+	    ndoc = new ApvlvDoc (this, type, w, h, gParams->values ("zoom"), false);
 	    if (!ndoc->loadfile (filename))
 	      {
 		delete ndoc;
@@ -602,8 +597,10 @@ namespace apvlv
 	std::vector < ApvlvCore * >::iterator itr = mDocs.begin ();
 	debug ("to pdf cache size: %d, remove first: %p\n",
 	       gParams->valuei ("pdfcache"), *itr);
+        delete *itr;
 	mDocs.erase (itr);
       }
+
     mDocs.push_back (core);
   }
 
@@ -715,7 +712,10 @@ namespace apvlv
 
     mCmdType = cmdtype;
 
-    mRootWindow->setsize (mWidth, adjheight ());
+    if (mRootWindow)
+      {
+        mRootWindow->setsize (mWidth, adjheight ());
+      }
 
     gtk_widget_show (mCommandBar);
     gtk_widget_grab_focus (mCommandBar);
@@ -729,7 +729,8 @@ namespace apvlv
     mCmdType = CMD_NONE;
 
     gtk_widget_hide (mCommandBar);
-    mRootWindow->setsize (mWidth, adjheight ());
+    if (mRootWindow)
+      mRootWindow->setsize (mWidth, adjheight ());
 
     gtk_widget_grab_focus (mMainWindow);
   }
@@ -833,6 +834,7 @@ namespace apvlv
   {
     guint procmd = mProCmd;
     mProCmd = 0;
+    mProCmdCnt = 0;
     switch (procmd)
       {
       case CTRL ('w'):
@@ -856,15 +858,21 @@ namespace apvlv
 	break;
 
       case 'g':
-	if (ct == 0)
-	  ct = 1;
-
-	if (key == 't')
-	  switchtab (mCurrTabPos + ct);
-	else if (key == 'T')
-	  switchtab (mCurrTabPos - ct);
-	else if (key == 'g')
-	  crtadoc ()->showpage (0);
+	if (key == 't'){
+	  if (ct == 0)
+	    switchtab (mCurrTabPos+1);
+	  else
+	    switchtab (ct-1);
+        }else if (key == 'T'){
+	  if (ct == 0)
+	    switchtab (mCurrTabPos-1);
+	  else
+	    switchtab (ct-1);
+        }else if (key == 'g'){
+	  if (ct == 0)
+	    ct = 1;
+	  crtadoc ()->showpage (ct-1);
+        }
 	break;
 
       default:
@@ -879,13 +887,14 @@ namespace apvlv
   {
     if (mProCmd != 0)
       {
-	return subprocess (ct, key);
+	return subprocess (mProCmdCnt, key);
       }
 
     switch (key)
       {
       case CTRL ('w'):
 	mProCmd = CTRL ('w');
+        mProCmdCnt = has ? ct : 0;
 	return NEED_MORE;
 	break;
       case 'q':
@@ -896,6 +905,7 @@ namespace apvlv
 	break;
       case 'g':
 	mProCmd = 'g';
+        mProCmdCnt = has ? ct : 0;
 	return NEED_MORE;
       default:
 	return crtadoc ()->process (has, ct, key);
@@ -982,7 +992,7 @@ namespace apvlv
 	  }
 	else if (cmd == "map" && subcmd != "")
 	  {
-	    gCmds->buildmap (subcmd.c_str (), argu.c_str ());
+            mCmds.buildmap (subcmd.c_str (), argu.c_str ());
 	  }
 	else if ((cmd == "o"
 		  || cmd == "open" || cmd == "doc") && subcmd != "")
@@ -1001,16 +1011,6 @@ namespace apvlv
 	      {
 		ret = loaddir (subcmd.c_str ());
 	      }
-#ifdef APVLV_WITH_HTML
-	    else if (g_ascii_strncasecmp (subcmd.c_str (), "http://", 7) == 0
-		     || g_ascii_strcasecmp (subcmd.c_str () + subcmd.length () - 4,
-					    ".htm") == 0
-		     || g_ascii_strcasecmp (subcmd.c_str () + subcmd.length () - 5,
-					    ".html") == 0)
-	      {
-		ret = loadhtml (subcmd.c_str ());
-	      }
-#endif
 	    else if (g_file_test (subcmd.c_str (), G_FILE_TEST_EXISTS))
 	      {
 		ret = loadfile (subcmd.c_str ());
@@ -1173,13 +1173,17 @@ namespace apvlv
     GtkAllocation allocation;
 
     gtk_widget_get_allocation(view->mViewBox, &allocation);
+    //debug ("view resized: %d:%d", allocation.width, allocation.height);
     w = allocation.width - 12;
     h = allocation.height - 12;
     if (w != view->mWidth || h != view->mHeight)
       {
 	view->mWidth = w;
 	view->mHeight = h;
-	view->mRootWindow->setsize (w, view->adjheight () - 10);
+        if (view->mRootWindow)
+          {
+            view->mRootWindow->setsize (w, view->adjheight () - 10);
+          }
       }
   }
 
@@ -1187,9 +1191,10 @@ namespace apvlv
   ApvlvView::apvlv_view_keypress_cb (GtkWidget * wid, GdkEvent * ev,
 				     ApvlvView * view)
   {
+    debug("view: %p, got key: %u", view, ((GdkEventKey* ) ev)->keyval);
     if (view->mCmdType == CMD_NONE)
       {
-	gCmds->append ((GdkEventKey *) ev);
+	view->mCmds.append ((GdkEventKey *) ev);
 	return TRUE;
       }
 
@@ -1322,8 +1327,21 @@ namespace apvlv
   ApvlvView::apvlv_view_delete_cb (GtkWidget * wid, GtkAllocation * al,
 				   ApvlvView * view)
   {
+    debug("delete cb: %p", view);
+    GtkWidget *widget = view->mMainWindow;
+    if (widget)
+      {
+        gtk_widget_destroy (widget);
+      }
     view->mMainWindow = NULL;
-    gtk_main_quit ();
+    if (view->mParent == NULL)
+      {
+        gtk_main_quit ();
+      }
+    else
+      {
+        view->mParent->back_tabcontext (0);
+      }
   }
 
   void
@@ -1402,6 +1420,26 @@ namespace apvlv
 				tabname);
     gtk_notebook_set_current_page (GTK_NOTEBOOK (mTabContainer), mCurrTabPos);
   }
+
+  void
+  ApvlvView::append_child (ApvlvView *view)
+  {
+    mChildren.push_back(view);
+  }
+
+  void
+  ApvlvView::erase_child (ApvlvView *view)
+  {
+    std::vector <ApvlvView *>::const_iterator itr = mChildren.begin();
+    while (*itr != view && itr != mChildren.end())
+      itr ++;
+
+    if (*itr == view)
+      {
+        mChildren.erase(itr);
+      }
+  }
+
 }
 
 // Local Variables:
